import axios from 'axios';
import { useAuth } from '../auth'; // Импортируем функцию авторизации
import { isAccountBlocked } from './socketService'; // Импортируем проверку блокировки из socketService

// Создаем экземпляр Axios с базовым URL нашего API
// Настраиваем базовый URL в зависимости от окружения

// Храним токены отмены запросов для возможности отмены при блокировке
const cancelTokenSources = new Map();

// Проверка блокировки аккаунта из localStorage
const checkAccountBlockStatus = () => {
  try {
    const blockedInfo = localStorage.getItem('accountBlockedInfo');
    if (blockedInfo) {
      const blockedData = JSON.parse(blockedInfo);
      if (blockedData?.blocked === true) {
        console.warn('[API] Обнаружен заблокированный аккаунт, запросы невозможны');
        return true;
      }
    }

    // Проверяем также флаг из socketService
    if (isAccountBlocked && isAccountBlocked()) {
      console.warn('[API] Аккаунт заблокирован согласно socketService, запросы невозможны');
      return true;
    }

    return false;
  } catch (e) {
    console.error('[API] Ошибка при проверке статуса блокировки:', e);
    return false;
  }
};

// Определяем базовый URL в зависимости от среды выполнения
const getBaseUrl = () => {
  // Всегда используем относительный путь /api.
  // В режиме разработки Vite dev server будет проксировать эти запросы согласно настройке server.proxy в vite.config.js.
  // В продакшене предполагается, что веб-сервер (например, Nginx) настроен аналогично для обработки /api.
  // return '/api';
  return import.meta.env.VITE_API_BASE_URL; // Используем переменную окружения Vite
};

const apiClient = axios.create({
  baseURL: getBaseUrl(),
  headers: {
    'Content-Type': 'application/json',
  },
});

// (Опционально, но очень полезно) Перехватчик для добавления JWT токена к запросам
apiClient.interceptors.request.use(
  (config) => {
    // Проверяем, заблокирован ли аккаунт, ПЕРЕД выполнением запроса
    if (checkAccountBlockStatus()) {
      // Если аккаунт заблокирован, отменяем запрос
      const source = axios.CancelToken.source();
      config.cancelToken = source.token;
      source.cancel('Запрос отменен из-за блокировки аккаунта');
      console.error('[API] Запрос отменен из-за блокировки аккаунта:', config.url);
      
      // Перенаправляем на страницу логина, если пользователь пытается выполнять запросы
      setTimeout(() => {
        if (window.location.pathname !== '/login') {
          window.location.href = '/login?blocked=true';
        }
      }, 0);
      
      return Promise.reject(new Error('Аккаунт заблокирован'));
    }
    
    const token = localStorage.getItem('userToken'); // Предполагаем, что токен хранится в localStorage
    if (token) {
      console.log('[API] Добавление токена к запросу:', config.url);
      config.headers['Authorization'] = `Bearer ${token}`;
    } else {
      console.log('[API] Токен не найден для запроса:', config.url);
    }
    
    // Добавляем CancelToken к запросу для возможности отмены
    const source = axios.CancelToken.source();
    config.cancelToken = source.token;
    
    // Сохраняем источник токена отмены в Map по URL запроса как ключу
    const requestKey = `${config.method}-${config.url}`;
    cancelTokenSources.set(requestKey, source);
    
    return config;
  },
  (error) => {
    console.error('[API] Ошибка в перехватчике запросов:', error);
    return Promise.reject(error);
  }
);

// (Опционально) Перехватчик ответов для обработки глобальных ошибок, например, 401
apiClient.interceptors.response.use(
  (response) => {
    // Удаляем источник токена отмены после успешного ответа
    const requestKey = `${response.config.method}-${response.config.url}`;
    cancelTokenSources.delete(requestKey);
    return response;
  },
  (error) => {
    // Удаляем источник токена отмены при ошибке
    if (error.config) {
      const requestKey = `${error.config.method}-${error.config.url}`;
      cancelTokenSources.delete(requestKey);
    }
    
    // Проверяем на блокировку аккаунта
    if (error.response && error.response.status === 403) {
      const errorData = error.response.data;
      // Проверка на признаки блокировки аккаунта в ответе
      if (errorData && (
          errorData.blocked === true || 
          errorData.message?.includes('заблокирован') || 
          errorData.message?.includes('blocked')
      )) {
        console.error('[API] Получено уведомление о блокировке аккаунта:', errorData);
        
        // Вызываем обработчик блокировки аккаунта
        setTimeout(() => {
          const { handleAccountBlocked } = useAuth();
          if (typeof handleAccountBlocked === 'function') {
            handleAccountBlocked(errorData);
          }
        }, 0);
      }
    }
    
    if (error.response && error.response.status === 401) {
      console.error('[API] Неавторизованный запрос (401):', error.config.url);
      
      // Обходим замыкание, чтобы избежать проблем с импортом
      // При использовании такого подхода, logout будет вызван после текущего цикла выполнения JS
      setTimeout(() => {
        const { logout } = useAuth();
        logout();
      }, 0);
    }
    return Promise.reject(error);
  }
);

// Функция для отмены всех активных запросов
const cancelActiveRequests = () => {
  console.log('[API] Отмена всех активных запросов. Количество: ' + cancelTokenSources.size);
  cancelTokenSources.forEach((source, key) => {
    source.cancel('Запрос отменен из-за блокировки аккаунта');
    console.log(`[API] Запрос ${key} отменен`);
  });
  cancelTokenSources.clear();
};

// Функция для явной установки токена аутентификации
const setAuthToken = (token) => {
  if (token) {
    apiClient.defaults.headers.common['Authorization'] = `Bearer ${token}`;
  } else {
    delete apiClient.defaults.headers.common['Authorization'];
  }
};

// Экспортируем функции для конкретных эндпоинтов
export default {
  // Добавляем новые утилитарные методы
  cancelActiveRequests,
  setAuthToken,
  checkAccountBlockStatus,
  
  register(userData) {
    return apiClient.post('/auth/register', userData);
  },
  login(credentials) {
    return apiClient.post('/auth/login', credentials);
  },
  getMe() { // Для получения данных пользователя
    return apiClient.get('/auth/me');
  },
  updateUserProfile(profileData) { 
    console.log('[API] Отправка запроса на обновление профиля:', profileData);
    return apiClient.put('/users/profile', profileData);
  },
  getSuggestions() {
    return apiClient.get('/users/suggestions');
  },
  likeUser(userId) {
    return apiClient.post(`/actions/like/${userId}`);
  },
  passUser(userId) {
    return apiClient.post(`/actions/pass/${userId}`);
  },
  uploadUserProfilePhoto(formData) {
    return apiClient.post('/users/profile/photo', formData, {
      headers: {
        'Content-Type': 'multipart/form-data', // Важно для FormData
      },
    });
  },
  // Новые методы для управления фотографиями
  setMainPhoto(photoId) {
    return apiClient.put(`/users/profile/photo/${photoId}/set-main`);
  },
  deletePhoto(photoId) {
    return apiClient.delete(`/users/profile/photo/${photoId}`);
  },
  getUserConversations() {
    return apiClient.get('/conversations'); // GET /api/conversations
  },
  getMessagesForConversation(conversationId) {
    return apiClient.get(`/conversations/${conversationId}/messages`); // GET /api/conversations/:id/messages
  },
  deleteMessage(conversationId, messageId) {
    return apiClient.delete(`/conversations/${conversationId}/messages/${messageId}`);
  },
  editMessage(conversationId, messageId, newText) {
    return apiClient.put(`/conversations/${conversationId}/messages/${messageId}`, { text: newText });
  },
  
  // Новый метод для получения профиля пользователя по ID
  getUserById(userId) {
    return apiClient.get(`/users/${userId}`);
  },
  
  // Новый метод для получения статистики пользователя
  getUserStats() {
    return apiClient.get('/users/stats');
  },
  
  // Новый метод для записи просмотра профиля
  recordProfileView(userId, source = 'other') {
    return apiClient.post(`/users/${userId}/view`, { source });
  },
  
  // Методы для работы с жалобами
  createReport(reportData) {
    return apiClient.post('/reports', reportData);
  },
  
  // Админские методы для жалоб
  getReports(params = {}) {
    return apiClient.get('/admin/reports', { params });
  },
  
  getReportById(reportId) {
    return apiClient.get(`/admin/reports/${reportId}`);
  },
  
  updateReportStatus(reportId, updateData) {
    return apiClient.put(`/admin/reports/${reportId}`, updateData);
  },
  
  getReportsStats() {
    return apiClient.get('/admin/reports/stats');
  }
};